searchData={"items":[{"type":"module","title":"EctoI18n","doc":"Provides i18n support for Ecto.","ref":"EctoI18n.html"},{"type":"module","title":"Preface - EctoI18n","doc":"There're [lots of strategies to localize contents in database](https://dejimata.com/2017/3/3/translating-with-mobility)\n([archived](https://web.archive.org/web/20240528023514/https://dejimata.com/2017/3/3/translating-with-mobility)).\nFor now, `EctoI18n` implements only strategy 6 mentioned\nabove - creating an extra column for storing all the localized contents\nfor that table.\n\nWith this strategy, it can:\n\n  * avoid using extra tables for storing localized contents.\n  * avoid using complex JOINs when retrieving localized contents.\n\n> Maybe other strategies will be implemented later, but for now, I only\n> need this one.","ref":"EctoI18n.html#module-preface"},{"type":"module","title":"Quick start - EctoI18n","doc":"Let's say that we have a schema which needs to be localized:\n\n    defmodule MyApp.Shop.Product do\n      use Ecto.Schema\n\n      schema \"products\" do\n        field :sku, :string\n        field :name, :string\n      end\n    end\n\nThe first step is to add a new column to the table at database level, so\nwe can store localized contents in it:\n\n    defmodule MyApp.Repo.Migrations.AddLocalesToProducts do\n      use Ecto.Migration\n\n      def change do\n        alter table(:products) do\n          add :locales, :map\n        end\n      end\n    end\n\nThe second step is to update schema for using the new column:\n\n    defmodule MyApp.Shop.Product do\n      use Ecto.Schema\n      use EctoI18n.Schema, locales: [\"en\", \"zh-Hans\", \"zh-Hant\"], default_locale: \"en\"\n\n      schema \"products\" do\n        field :sku, :string\n        field :name, :string\n\n        locales :locales do\n          field :name, :string\n        end\n      end\n    end\n\n> If you're curious about the underlying implementation here, you can read\n> `EctoI18n.Schema.locales/2` to learn more.\n\nNext, you can use the extensions provided by `EctoI18n` to\nwork with the localized schema, such as:\n\n  * `EctoI18n.localize!/2`\n  * `EctoI18n.Changeset.cast_locales/3`\n  * `EctoI18n.Query` (Still in planning)\n  * ...","ref":"EctoI18n.html#module-quick-start"},{"type":"function","title":"EctoI18n.localizable!/1","doc":"Ensures that a module or a struct is localizable. Or, an error is raised.","ref":"EctoI18n.html#localizable!/1"},{"type":"function","title":"EctoI18n.localizable?/1","doc":"Checks whether a module or a struct is localizable.","ref":"EctoI18n.html#localizable?/1"},{"type":"function","title":"Examples - EctoI18n.localizable?/1","doc":"iex> EctoI18n.localizable?(Product)\n    iex> EctoI18n.localizable?(%Product{})","ref":"EctoI18n.html#localizable?/1-examples"},{"type":"function","title":"EctoI18n.localizable?/2","doc":"Checks whether a field in a module or a struct is localizable.","ref":"EctoI18n.html#localizable?/2"},{"type":"function","title":"Examples - EctoI18n.localizable?/2","doc":"iex> Ecto.localizable?(Product, :name)\n    iex> Ecto.localizable?(%Product{}, :name)","ref":"EctoI18n.html#localizable?/2-examples"},{"type":"function","title":"EctoI18n.localize!/2","doc":"Localizes a struct with given locale recursively.\n\nAll localizable values in the struct will be localized into the give locale.","ref":"EctoI18n.html#localize!/2"},{"type":"function","title":"Examples - EctoI18n.localize!/2","doc":"iex> EctoI18n.localize!(product, \"zh-Hans\")","ref":"EctoI18n.html#localize!/2-examples"},{"type":"type","title":"EctoI18n.locale/0","doc":"","ref":"EctoI18n.html#t:locale/0"},{"type":"module","title":"EctoI18n.Changeset","doc":"Provides i18n extensions for `Ecto.Changeset`.","ref":"EctoI18n.Changeset.html"},{"type":"function","title":"EctoI18n.Changeset.cast_locales/3","doc":"Casts field which is added by `locales/2` macro.\n\n> The `:with` option is always required.\n>\n> `EcotI18n.Schema` doesn't generate any default `changeset/2` function\n> for the generated schemas. This is intentional. Because no matter what\n> I do, some needs will be hard to meet, so it's better to leave the\n> specific implementation to the developer.\n\n> It is built on the top of `Ecto.Changeset.cast_embed/3`.","ref":"EctoI18n.Changeset.html#cast_locales/3"},{"type":"function","title":"Example - EctoI18n.Changeset.cast_locales/3","doc":"defmodule MyApp.Shop.Product do\n      use Ecto.Schema\n      import Ecto.Changeset\n      use EctoI18n.Schema, locales: [\"en\", \"zh-Hans\", \"zh-Hant\"], default_locale: \"en\"\n      import EctoI18n.Changeset\n\n      schema \"products\" do\n        field :sku, :string\n        field :name, :string\n\n        locales :locales do\n          field :name, :string\n        end\n      end\n\n      def changeset(product, attrs) do\n        product\n        |> cast(attrs, [:sku, :name])\n        |> cast_locales(:locales, with: &cast_locale/2)\n      end\n\n      def cast_locale(locale, attrs) do\n        locale\n        |> cast(attrs, [:name])\n        |> validate_required([:name])\n      end\n    end\n\nIn above code, `changeset/2` equals to:\n\n    def changeset(product, attrs) do\n      product\n      |> cast(attrs, [:sku, :name])\n      |> cast_embed(:locales, required: true, with: &cast_locales/2)\n    end\n\n    defp cast_locales(locales, attrs) do\n      locales\n      |> cast(attrs, [])\n      |> cast_embed(:\"zh-Hans\", required: true, with: &cast_locale/2)\n      |> cast_embed(:\"zh-Hant\", required: true, with: &cast_locale/2)\n    end","ref":"EctoI18n.Changeset.html#cast_locales/3-example"},{"type":"module","title":"EctoI18n.Query","doc":"Provides i18n extensions for `Ecto.Query`.","ref":"EctoI18n.Query.html"},{"type":"module","title":"Plans - EctoI18n.Query","doc":"I plan to build this module like `Cldr.Trans.QueryBuilder`.\n\n  * https://github.com/elixir-cldr/cldr_trans/blob/main/lib/cldr/trans/query_builder.ex\n  * https://github.com/elixir-cldr/cldr_trans/blob/main/lib/cldr/trans/gen_function_migration.ex","ref":"EctoI18n.Query.html#module-plans"},{"type":"module","title":"EctoI18n.Schema","doc":"Provides i18n extensions for `Ecto.Schema`.","ref":"EctoI18n.Schema.html"},{"type":"macro","title":"EctoI18n.Schema.locales/2","doc":"Creates a field for storing localized contents.","ref":"EctoI18n.Schema.html#locales/2"},{"type":"macro","title":"Example - EctoI18n.Schema.locales/2","doc":"defmodule MyApp.Shop.Product do\n      use Ecto.Schema\n      use EctoI18n.Schema, locales: [\"en\", \"zh-Hans\", \"zh-Hant\"], default_locale: \"en\"\n\n      schema \"products\" do\n        field :sku, :string\n        field :name, :string\n\n        locales :locales do\n          field :name, :string\n        end\n      end\n    end\n\nIn above code, calling:\n\n    locales :locales do\n      field :name, :string\n    end\n\nequals to:\n\n    embeds_one :locales, Locales do\n      embeds_one :\"zh-Hans\", Fields\n      embeds_one :\"zh-Hant\", Fields\n    end\n\nThe `Locales` and `Fields` modules will be generated automatically.","ref":"EctoI18n.Schema.html#locales/2-example"},{"type":"extras","title":"EctoI18n","doc":"# EctoI18n\n\n> Provides i18n support for Ecto.","ref":"readme.html"},{"type":"extras","title":"Notes - EctoI18n","doc":"This package is still in its early stages, so it may still undergo significant changes, potentially leading to breaking changes.","ref":"readme.html#notes"},{"type":"extras","title":"Installation - EctoI18n","doc":"Add `:ecto_i18n` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_i18n,  }\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - EctoI18n","doc":"For more information, see the [documentation](https://hexdocs.pm/ecto_i18n).","ref":"readme.html#usage"},{"type":"extras","title":"Terminology - EctoI18n","doc":"Throughout the package, I will use the following terminology to avoid conceptual confusion.","ref":"readme.html#terminology"},{"type":"extras","title":"g11n (globalization) - EctoI18n","doc":"It is a broader concept that covers the i18n and l10n processes throughout the software development lifecycle.\n\nThe goal of g11n is to make products adapted in different languages and cultural environments:\n\n- i18n enables products to be adaptable.\n- l10n lets products to be adapted.","ref":"readme.html#g11n-globalization"},{"type":"extras","title":"i18n (internationalization) - EctoI18n","doc":"It refers to the process of designing and developing a product to be adaptable to different languages and cultural environments.\n\nIt is the preparation for locaization. When i18n is done, there's no need for further programmatic changes to the product to switch between languages.\n\nAt this stage, we are not conducting actual l10n work but rather providing the infrastructure for l10n.","ref":"readme.html#i18n-internationalization"},{"type":"extras","title":"l10n (localization) - EctoI18n","doc":"It refers to the process of adapting a product to a specific language or region. It includes:\n\n- translating text.\n- adjusting formats, such as date, time, number, currencies, etc.\n- ...\n\nAt this stage, we focus on the actual l10n work.","ref":"readme.html#l10n-localization"},{"type":"extras","title":"Thanks - EctoI18n","doc":"This package is built on the wisdom of:\n\n- [trans](https://github.com/crbelaus/trans)\n- [cldr_trans](https://github.com/elixir-cldr/cldr_trans)","ref":"readme.html#thanks"},{"type":"extras","title":"License - EctoI18n","doc":"[Apache License 2.0](http://www.apache.org/licenses/LICENSE-2.0)","ref":"readme.html#license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,51,46,48]}}